#ifndef ANNOTATIONS_HPP_
#define ANNOTATIONS_HPP_

#include <string>
#include <map>

#include <boost/shared_ptr.hpp>

#include "mussa_exceptions.hpp"

class Annotations;
typedef boost::shared_ptr<Annotations> AnnotationsRef;

/**
 * \brief Annotations provides a simple "dictionary" of name value pairs
 * 
 * The intent is that this is to be used as a mix-in class to add
 * a uniform way of storing annotation metadata.
 */
class Annotations {
protected:
  static const std::string name_str;

public:
  typedef std::map<std::string, std::string> metadata_map;
  typedef metadata_map::size_type size_type;
  
  Annotations();
  Annotations(const std::string& n);
  Annotations(const Annotations&);
  Annotations(const AnnotationsRef);

  //! make a shared pointer copy of our object
  AnnotationsRef copy() const;
    
  //! provide a special case for accessing a name
  std::string name() const { return metadata.find(name_str)->second; }
  //! special case for setting name
  void setName(const std::string& n) { metadata[name_str] = n; }
  
  //! Generic metadata handling
  //@{
  //! remove some key from our "dictionary"
  void erase(const std::string& key);  
  //! set a metadata element 
  void set(const std::string& key, const std::string &value );
  //! look for a key, \throws annotation_key_error if it doesn't find it
  std::string get(const std::string& key) const;
  //! check for a key
  bool has_key(const std::string& key) const;
  //! return all the keys of our metadata map
  //std::list<std::string> keys() const;
  size_type size() const { return metadata.size(); }
  //@}
protected:
  metadata_map metadata;
};
#endif /*ANNOTATIONS_HPP_*/
