#ifndef _TRIVIAL_FILTER_HPP_
#define _TRIVIAL_FILTER_HPP_

#include <vector>

//! symbolic constants used to inidicate numeric values for nucleotides
/*! the ordering comes from the very simple bit-wise operator
 *  used to convert ascii values to the range 0..7
 */
enum nucleotide { X, A, C=3, T, U, N, G };

//! convert a nucleotide ascii encoding to value in the range 0..7
inline enum nucleotide nt_map(char c) 
{
  return static_cast<nucleotide>(c & 0x07);
}

//! generates our transition table
class TSFLookupTable {
  public:
    //! create transition table using DNA symbols
    /*! generate all the dna patterns of n base pairs and 
     * require length copies of the pattern.
     *
     * e.g. n=2, length=2 would match things like
     * ATAT *(AT), 
     */

    TSFLookupTable(int n, int length);

    //! return the next state
    int next(int current, enum nucleotide symbol);

    //! how many states are there in our table?
    int states() { return table.size(); }
   
    static const int start_state = 0;
    static const int accept_state = -1;
    static const int failed_state = -2;

  private:
    typedef std::vector<enum nucleotide> symbol_vec;

    //! generate length copies of a pattern n bases long
    void generate_dna_pattern(int n, int length);

    std::vector<std::vector<int> > table;

};

class TrivialSequenceFilter {
  public:
  private:
    TSFLookupTable table;
};


#endif // _TRIVIAL_FILTER_HPP_
