#include <QSlider>
#include <QString>

#include "qui/ThresholdWidget.hpp"
#include <iostream>
#include <math.h>

// This is completely and totally derived from the Qt example
// LCDRange.cpp
ThresholdWidget::ThresholdWidget(QWidget *parent, int min, int max) 
  : QWidget(parent),
    layout(0),
    basepair_spinner(0),
    window_size_label(0),
    percent_label(0)
{
  setupWidgets();
  basepair_spinner->setWhatsThis("How many base pairs need to be conserved in the window size");

  setRange(min, max);

  connect(basepair_spinner, SIGNAL(valueChanged(int)), 
          this, SLOT(setBasepairThreshold(int)));

  basepair_spinner->setValue(min);
  updatePercentThreshold();

  layout->addWidget(new QLabel("Threshold"));
  layout->addWidget(basepair_spinner);
  layout->addWidget(window_size_label);
  layout->addWidget(percent_label);
  setLayout(layout);
}

void ThresholdWidget::setupWidgets()
{
  layout = new QHBoxLayout(this);
  basepair_spinner = new QSpinBox(this);
  window_size_label = new QLabel(this);
  percent_label = new QLabel(this);
}
  
void ThresholdWidget::setRange(int min, int max)
{
  basepair_spinner->setRange(min, max);
  cur_bp_threshold = min;
  basepair_spinner->setValue(min);

  QString num;
  num.setNum(max);
  window_size_label->setText("/"+num);
  updatePercentThreshold();
}

bool ThresholdWidget::isReadOnly() const
{
  return basepair_spinner->isReadOnly();
}

void ThresholdWidget::setReadOnly(bool read_only)
{
  basepair_spinner->setReadOnly(read_only);
}

double ThresholdWidget::min_ratio() const
{
  double ratio=((double)basepair_spinner->minimum()/
                (double)basepair_spinner->maximum());

  return ratio;
}

double ThresholdWidget::ratio() const
{
  return ((float)threshold())/((float)basepair_spinner->maximum()); 
}

void ThresholdWidget::setBasepairThreshold(int threshold)
{
 if (cur_bp_threshold!= threshold)
  {
    cur_bp_threshold = threshold;
    cur_percent_threshold = (int)(roundl(ratio() * 100));
    basepair_spinner->setValue(threshold);
    updatePercentThreshold();
    emit thresholdChanged(threshold);
  }
}

void ThresholdWidget::updatePercentThreshold()
{
  int min_percent = (int)fmaxl(0.0, roundl(ratio()*100));
  QString label;
  label.setNum(min_percent);
  label += "%";
  percent_label->setText(label);
}

int ThresholdWidget::threshold() const
{
  return cur_bp_threshold;
}
