#ifndef _SEQUENCE_PROPERTIES_MODEL_HPP_
#define _SEQUENCE_PROPERTIES_MODEL_HPP_

#include <QObject>
#include <QAbstractTableModel>
#include <vector>

#include "alg/glsequence.hpp"

class SequencePropertiesModel : public QAbstractTableModel
{
  Q_OBJECT

  public:
    typedef boost::shared_ptr<GlSequence> model_item;
    typedef std::vector<model_item> model_type;
    typedef model_type::size_type size_type;
    typedef model_type::iterator iterator; 
    typedef model_type::const_iterator const_iterator; 

    SequencePropertiesModel(model_type model, QObject *parent = 0);

    //! \defgroup VectorInterface
    //! \addtogroup VectorInterface
    //! \@{
    //! assign num copies of val to our vector
    void assign(size_type num, const model_item val);
    //! return a specific element
    model_item at(size_type index);
    //! return the last element
    model_item back();
    //! return an iterator to the beginning of the model
    iterator begin();
    //! return an iterator to the beginning of the model
    const_iterator begin() const;
    //! empty the model
    void clear();
    //! return an iterator to the end of the model
    iterator end();
    //! return an iterator to the end of the model
    const_iterator end() const;
    //! is the model empty?
    bool empty() const;
    //! return a specific element
    model_item operator[](size_type index);
    //! remove the last element from our model
    void pop_back();
    //! add a sequence to the end of our model
    void push_back(model_item);
    //! how many elements are in our model.
    size_type size() const;
    //! \@}

    //! \defgroup QtModelInterface
    //! \addtogroup QtModel
    //! \@{
    int rowCount(const QModelIndex& parent=QModelIndex()) const;
    int columnCount(const QModelIndex& parent=QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role=Qt::DisplayRole) const;

    QVariant headerData(int section, Qt::Orientation orientation, int role=Qt::DisplayRole) const;

    bool setData(const QModelIndex& index, const QVariant &value, 
                 int role=Qt::EditRole);
    Qt::ItemFlags flags(const QModelIndex& index) const;

    //bool insertRows(int row, int count, 
    //                const QModelIndex& parent=QModelIndex());
    //bool removeRows(int row, int count,
    //                const QModelIndex& parent=QModelIndex());
    //! \@}

  public slots:
    //! update the original sequence with our changes    
    void saveChanges();
    
  private:
    model_type sequences;
    model_type original_sequences;
};

#endif
